import { IConfig } from '@zeedhi/core';

export interface PmeMask extends Object {
  pattern: string;
  nivelMax?: number;
}

export interface PmeCurrencyMask extends Object {
  currencySymbol: string;
}

const cdpictprod: string|null = localStorage.getItem('CDPICTPROD');
const cdpictprat: string|null = localStorage.getItem('CDPICTPRAT');
const displayFormat = localStorage.getItem('DATEFORMAT') || undefined;
const currencySymbol = localStorage.getItem('CURRENCY') || undefined;

const cnpjMask: PmeMask = {
  pattern: 'NN.NNN.NNN/NNNN-NN',
};

const cpfMask: PmeMask = {
  pattern: 'NNN.NNN.NNN-NN',
};

const productMask: PmeMask = {
  pattern: cdpictprod ? cdpictprod.replace(/[0-9a-zA-Z!@#$%^&*()_+]/g, 'N') : 'N.NN.NN.NNN.NN',
  nivelMax: cdpictprod ? cdpictprod.split('.').length : 5,
};

const plateMask: PmeMask = {
  pattern: cdpictprat ? cdpictprat.replace(/[0-9a-zA-Z!@#$%^&*()_+]/g, 'N') : 'NN.NN.NN.NNN',
  nivelMax: cdpictprat ? cdpictprat.split('.').length : 4,
};

const config: IConfig = {
  language: 'pt-BR',
  dateFormat: 'YYYY-MM-DD',
  displayFormat,
  displayDateTimeFormat: 'DD/MM/YYYY HH:mm',
  dateTimeFormat: 'YYYY-MM-DDTHH:mm:ss',
  homeUrl: '/',
  monthFormat: 'MM/YY',
  monthDisplayFormat: 'MMM/YY',
  endPoint: process.env.VUE_APP_END_POINT,
  metadataEndPoint: process.env.VUE_APP_METADATA_END_POINT,
  env: {
    errorImagePath: `${process.env.VUE_APP_PUBLIC_PATH}img/`,
    loginBackground: process.env.VUE_APP_LOGIN_IMG,
    logo: process.env.VUE_APP_LOGO_IMG,
  },
  masks: {
    numberMask: {
      decimalCharacter: ',',
      decimalCharacterAlternative: '.',
      digitGroupSeparator: '.',
      decimalPlaces: 2,
      currencySymbol,
    },
    currencyMask: {
      decimalCharacter: ',',
      decimalCharacterAlternative: '.',
      digitGroupSeparator: '.',
      currencySymbol,
    },
    cnpjMask,
    cpfMask,
    productMask,
    plateMask,
  },
};

export default config;
