import {
  Metadata, Http, Loader, IEventParam, IDictionary,
} from '@zeedhi/core';

import {
  Select, SelectMultiple, Form, Modal, ModalService, Button,
} from '@zeedhi/common';
import { default as moment } from 'dayjs';

export class Pme00510AtulizarComensaisCardController {
  public appController = Loader.getInstance('AppController');
  
  public modalController = Loader.getInstance('ModalController');
  
  public utilities = Loader.getInstance('Utilities');
  
  public gridEditController = Loader.getInstance('GridEditableController');
  
  public dataProcessamentoDe?: string = '';
  
  public dataProcessamentoAte?: string = '';
  
  public dtProcessa?: string = '';
  
  private modalConfigCard!: Modal;
  
  public async createPeriodoModal() {
    this.modalController.createModal('/modals/formAtualizarComensais');
    this.modalConfigCard = await ModalService.createFromJson('modals/closeButtonModal', true);
  }
  
  public destroyPeriodoModal() {
    this.modalController.destroyModal();
  }
  
  public cancelModal() {
    this.modalConfigCard.hide();
    this.modalConfigCard.destroy();
    this.modalController.closeModal();
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const cardapio = Metadata.getInstance<Select>('TIPOCARD');
    const servico = Metadata.getInstance<Select>('SERVICO');
    unidade.setValue('');
    cardapio.setValue('');
    servico.setValue('');
  }
  
  public openModalAtualizaComensais() {
    if (this.utilities.centralIsValid()) {
      this.modalController.openModal();
      this.dataProcessamentoDe = this.dtProcessa;
      this.dataProcessamentoAte = this.dtProcessa;
    }
  }
  
  public async getDtProcessa() {
    this.dtProcessa = await this.utilities.getProcessDate();
    this.utilities.centralIsValid();
  }
  
  public setFilterUnidade({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public setFilterTpCardapio({ component }: IEventParam<SelectMultiple>) {
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterServ({ component }: IEventParam<SelectMultiple>) {
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const tipocard = Metadata.getInstance<Select>('TIPOCARD');

    let tipoCard = 'T';
    let cdfilial = 'T';

    if (unidade.value.length > 0) cdfilial = unidade.value.join('_');
    if (tipocard.value.length > 0) tipoCard = tipocard.value.join('_');

    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: tipoCard,
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdfilial,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public exportStoresDiners({ component }: IEventParam<Button>) {
    const form: Form = Metadata.getInstance('form');
    const row = form.value;
    row.CDFILCENTPME = this.appController.getCdCentral();
    if (this.validateDate(form)) {
      this.utilities.activeLoadingButton(component);
      Http.post('/buscaComensaisUnidade', { requestType: 'Row', row }).then(() => {
        Http.post('/buscaTotalCentral', { requestType: 'Row', row }).then(() => {
          this.utilities.showAlert('Alert', 'pme::cardAtualizadoSucesso', 'success');
        });
      }).finally(() => {
        this.utilities.disableLoadingButton(component);
      });
    }
  }
  
  private validateDate(form: Form) {
    let valid = false;
    const row: IDictionary = form.value;
    const periodoDe = row.PERIODODE;
    const periodoAte = row.PERIODOATE;
    if (moment(moment(periodoDe).format('L')) < moment(moment(this.dtProcessa).format('L'))) {
      this.utilities.showAlert('alertError', 'pme::dataIniMemorProcesamento', 'error');
    } else if (form.validate()) {
      if (this.utilities.dateIsValid(periodoDe, periodoAte)) valid = true;
      else this.utilities.showAlert('alertError', 'pme::dataIniMaior', 'error');
    }
    return valid;
  }

  public clearFieldServ() {
    const servico = Metadata.getInstance<Select>('SERVICO');
    servico.setValue([]);
  }
}
